<?php
/**
 * Gutentor Custom Fonts
 *
 * @since 1.0.0
 */

if ( ! class_exists( 'Gutentor_Fonts' ) ) {
	/**
	 * Class Gutentor_Fonts.
	 */
	class Gutentor_Fonts {

		public $post_type = 'gutentor-fonts';

		/**
		 * Main Gutentor_Fonts Instance
		 * Initialize the class and set its properties.
		 *
		 * @since    1.0.0
		 * @return object $instance Gutentor_Fonts Instance
		 */
		public static function instance() {

			// Store the instance locally to avoid private static replication
			static $instance = null;

			// Only run these methods if they haven't been ran previously
			if ( null === $instance ) {
				$instance = new Gutentor_Fonts();
			}

			// Always return the instance
			return $instance;
		}


		public function run() {
			add_filter( 'wp_check_filetype_and_ext', array( $this, 'filetype_and_ext' ), 10, 3 );
			add_filter( 'upload_mimes', array( $this, 'add_fonts_to_allowed_mimes' ) );
			add_action( 'init', array( $this, 'register_post_type_gutentor_fonts' ) );
			add_action( 'admin_menu', array( $this, 'add_post_type_submenu' ) );
			add_action( 'admin_head', array( $this, 'dynamic_fonts' ), 98 );
			add_action( 'wp_head', array( $this, 'dynamic_fonts' ), 98 );
			add_filter( 'gutentor_block_editor_localize_data', array( $this, 'add_localize_data' ), 98 );
			if ( is_admin() ) {
				add_action( 'load-post.php', array( $this, 'init_metabox' ) );
				add_action( 'load-post-new.php', array( $this, 'init_metabox' ) );
			}
		}

		/**
		 * Correct the mime types and extension for the font types.
		 *
		 * @param array  $defaults File data array containing 'ext', 'type', and
		 *                                          'proper_filename' keys.
		 * @param string $file                      Full path to the file.
		 * @param string $filename                  The name of the file (may differ from $file due to
		 *                                          $file being in a tmp directory).
		 * @return Array File data array containing 'ext', 'type', and
		 */
		public function filetype_and_ext( $defaults, $file, $filename ) {
			if ( 'ttf' === pathinfo( $filename, PATHINFO_EXTENSION ) ) {
				$defaults['type'] = 'application/x-font-ttf';
				$defaults['ext']  = 'ttf';
			}

			if ( 'otf' === pathinfo( $filename, PATHINFO_EXTENSION ) ) {
				$defaults['type'] = 'application/x-font-otf';
				$defaults['ext']  = 'otf';
			}

			return $defaults;
		}

		/**
		 * Allowed mime types and file extensions
		 *
		 * @since 1.0.0
		 * @param array $mimes Current array of mime types.
		 * @return array $mimes Updated array of mime types.
		 */
		public function add_fonts_to_allowed_mimes( $mimes ) {
			$mimes['woff']  = 'application/x-font-woff';
			$mimes['woff2'] = 'application/x-font-woff2';
			$mimes['ttf']   = 'application/x-font-ttf';
			$mimes['svg']   = 'image/svg+xml';
			$mimes['eot']   = 'application/vnd.ms-fontobject';
			$mimes['otf']   = 'font/otf';
			return $mimes;
		}

		public function set_dynamic_fonts_transists() {
			$args      = array(
				'post_type'   => $this->post_type,
				'post_status' => 'publish',
			);
			$the_query = new WP_Query( $args );

			$font_css = '';
			$add_data = array();
			if ( $the_query->have_posts() ) {
				$add_data['dynamic-font-types'] = array();

				while ( $the_query->have_posts() ) {
					$the_query->the_post();
					$font_fallback   = get_post_meta( get_the_ID(), 'gutentor-font-fallback', true );
					$font_display    = get_post_meta( get_the_ID(), 'gutentor-font-display', true );
					$font_variations = get_post_meta( get_the_ID(), 'gutentor-font-variation', true );
					if ( is_array( $font_variations ) ) {
						foreach ( $font_variations as $font_variation ) {
							$font_variation = array_merge(
								array(
									'weight' => 400,
									'style'  => 'normal',
									'woff'   => '',
									'woff2'  => '',
									'ttf'    => '',
									'svg'    => '',
									'eot'    => '',
									'otf'    => '',
								),
								$font_variation
							);
							$font_files     = array();
							if ( isset( $font_variation['woff'] ) && ! empty( $font_variation['woff'] ) ) {
								$font_files[] = 'url(' . esc_url( $font_variation['woff'] ) . ") format('woff')";
							}
							if ( isset( $font_variation['woff2'] ) && ! empty( $font_variation['woff2'] ) ) {
								$font_files[] = 'url(' . esc_url( $font_variation['woff2'] ) . ") format('woff2')";
							}
							if ( isset( $font_variation['ttf'] ) && ! empty( $font_variation['ttf'] ) ) {
								$font_files[] = 'url(' . esc_url( $font_variation['ttf'] ) . ") format('TrueType')";
							}
							if ( isset( $font_variation['svg'] ) && ! empty( $font_variation['svg'] ) ) {
								$font_files[] = 'url(' . esc_url( $font_variation['svg'] ) . ") format('svg')";
							}
							if ( isset( $font_variation['eot'] ) && ! empty( $font_variation['eot'] ) ) {
								$font_files[] = 'url(' . esc_url( $font_variation['eot'] ) . ") format('eot')";
							}
							if ( isset( $font_variation['otf'] ) && ! empty( $font_variation['otf'] ) ) {
								$font_files[] = 'url(' . esc_url( $font_variation['otf'] ) . ") format('OpenType')";
							}
							if ( ! empty( $font_files ) ) {
								$add_data['dynamic-font-types'][ get_the_title() ][] = $font_variation['weight'];

								$font_css   .= '@font-face {';
								$font_family = '"' . get_the_title() . '"';
								if ( $font_fallback ) {
									$font_family .= ', ' . $font_fallback;
								}
								$font_css .= 'font-family: ' . $font_family . ';';

								if ( $font_display ) {
									$font_css .= 'font-display: ' . $font_display . ';';
								}
								if ( ! isset( $font_variation['style'] ) || ! $font_variation['style'] ) {
									$font_variation['style'] = 'normal';
								}
								$font_css .= 'font-style: ' . $font_variation['style'] . ';';
								$font_css .= 'font-weight: ' . $font_variation['weight'] . ';';

								$font_css .= 'src: ' . implode( ', ', $font_files ) . ';';

								$font_css .= '} ';
							}
						}
					}
				}
			}

			if ( ! empty( $add_data ) ) {
				set_transient( 'gutentor_dynamic_fonts', $add_data );
				set_transient( 'gutentor_dynamic_fonts_css', $font_css );
			}
			wp_reset_postdata();
		}

		public function add_localize_data( $localize_data ) {
			if ( get_transient( 'gutentor_dynamic_fonts' ) ) {
				$localize_data['dynamic-fonts'] = get_transient( 'gutentor_dynamic_fonts' );
			}
			return $localize_data;
		}

		public function dynamic_fonts() {
			if ( get_transient( 'gutentor_dynamic_fonts_css' ) ) {
				?>
				<style type="text/css">
					<?php echo wp_strip_all_tags( get_transient( 'gutentor_dynamic_fonts_css' ) ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
				</style>
				<?php
			}
		}
		/**
		 * Register the register_post_type.
		 *
		 * @since    1.0.0
		 */
		public function add_post_type_submenu() {
			add_submenu_page(
				'gutentor',
				_x( 'All Fonts', 'Post type general name', 'gutentor' ),
				_x( 'Fonts', 'Post type general name', 'gutentor' ),
				'manage_options',
				'edit.php?post_type=gutentor-fonts'
			);
		}

		/**
		 * Register the register_post_type.
		 *
		 * @since    1.0.0
		 */
		public function register_post_type_gutentor_fonts() {

			$labels = apply_filters(
				'gutentor_post_type_fonts_label',
				array(
					'name'                  => _x( 'Fonts', 'Post type general name', 'gutentor' ),
					'singular_name'         => _x( 'Font', 'Post type singular name', 'gutentor' ),
					'menu_name'             => _x( 'Fonts', 'Admin Menu text', 'gutentor' ),
					'name_admin_bar'        => _x( 'Font', 'Add New on Toolbar', 'gutentor' ),
					'add_new'               => esc_html__( 'Add New', 'gutentor' ),
					'add_new_item'          => esc_html__( 'Add New Font', 'gutentor' ),
					'new_item'              => esc_html__( 'New Font', 'gutentor' ),
					'edit_item'             => esc_html__( 'Edit Font', 'gutentor' ),
					'view_item'             => esc_html__( 'View Font', 'gutentor' ),
					'all_items'             => esc_html__( 'All Fonts', 'gutentor' ),
					'search_items'          => esc_html__( 'Search Fonts', 'gutentor' ),
					'parent_item_colon'     => esc_html__( 'Parent Fonts:', 'gutentor' ),
					'not_found'             => esc_html__( 'No documentations found.', 'gutentor' ),
					'not_found_in_trash'    => esc_html__( 'No documentations found in Trash.', 'gutentor' ),
					'featured_image'        => _x( 'Font Cover Image', 'Overrides the “Featured Image” phrase for this post type. Added in 4.3', 'gutentor' ),
					'set_featured_image'    => _x( 'Set cover image', 'Overrides the “Set featured image” phrase for this post type. Added in 4.3', 'gutentor' ),
					'remove_featured_image' => _x( 'Remove cover image', 'Overrides the “Remove featured image” phrase for this post type. Added in 4.3', 'gutentor' ),
					'use_featured_image'    => _x( 'Use as cover image', 'Overrides the “Use as featured image” phrase for this post type. Added in 4.3', 'gutentor' ),
					'archives'              => _x( 'Font archives', 'The post type archive label used in nav menus. Default “Post Archives”. Added in 4.4', 'gutentor' ),
					'insert_into_item'      => _x( 'Insert into documentation', 'Overrides the “Insert into post”/”Insert into page” phrase (used when inserting media into a post). Added in 4.4', 'gutentor' ),
					'uploaded_to_this_item' => _x( 'Uploaded to this documentation', 'Overrides the “Uploaded to this post”/”Uploaded to this page” phrase (used when viewing media attached to a post). Added in 4.4', 'gutentor' ),
					'filter_items_list'     => _x( 'Filter documentations list', 'Screen reader text for the filter links heading on the post type listing screen. Default “Filter posts list”/”Filter pages list”. Added in 4.4', 'gutentor' ),
					'items_list_navigation' => _x( 'Fonts list navigation', 'Screen reader text for the pagination heading on the post type listing screen. Default “Posts list navigation”/”Pages list navigation”. Added in 4.4', 'gutentor' ),
					'items_list'            => _x( 'Fonts list', 'Screen reader text for the items list heading on the post type listing screen. Default “Posts list”/”Pages list”. Added in 4.4', 'gutentor' ),
				)
			);

			$args = apply_filters(
				'gutentor_post_type_fonts_args',
				array(
					'labels'             => $labels,
					'public'             => false,
					'publicly_queryable' => false,
					'show_ui'            => true,
					'show_in_menu'       => false,
					'query_var'          => true,
					'show_in_rest'       => true,
					'rewrite'            => false,
					'capability_type'    => 'post',
					'has_archive'        => false,
					'hierarchical'       => false,
					'menu_position'      => 999,
					'menu_icon'          => 'dashicons-admin-site',
					'supports'           => array( 'title', 'author' ),
				)
			);

			register_post_type( $this->post_type, $args );

			/*
			register_post_meta(
				$this->post_type,
				'gutentor-font-fallback',
				array(
					'show_in_rest' => true,
					'single'       => true,
					'type'         => 'string',
				)
			);
			register_post_meta(
				$this->post_type,
				'gutentor-font-display',
				array(
					'show_in_rest' => true,
					'single'       => true,
					'type'         => 'string',
				)
			);
			register_post_meta(
				$this->post_type,
				'gutentor-font-variation',
				array(
					'single'       => true,
					'type'         => 'array',
					'show_in_rest' => array(
						'schema' => array(
							'items' => array(
								'type'       => 'object',
								'properties' => array(
									'weight' => array(
										'type' => 'string',
									),
									'style'  => array(
										'type' => 'string',
									),
									'woff'   => array(
										'type' => 'string',
									),
									'woff2'  => array(
										'type' => 'string',
									),
									'ttf'    => array(
										'type' => 'string',
									),
									'svg'    => array(
										'type' => 'string',
									),
									'eot'    => array(
										'type' => 'string',
									),
									'otf'    => array(
										'type' => 'string',
									),
								),
							),
						),
					),
				)
			);*/
		}

		/**
		 * Meta box initialization.
		 */
		public function init_metabox() {

			add_action( 'add_meta_boxes', array( $this, 'add_metabox' ) );
			add_action( 'save_post', array( $this, 'save_metabox' ), 10, 2 );
		}

		/**
		 * Adds the meta box.
		 * On fonts
		 */
		public function add_metabox() {
			add_meta_box(
				'gutentor_meta_box',
				__( 'Font Info', 'gutentor' ),
				array( $this, 'render_metabox' ),
				$this->post_type,
				'normal',
				'high'
			);
		}

		/**
		 * Renders the meta box.
		 */
		public function render_metabox( $post ) {

			/*Item Font URL*/
			$font_fallback   = get_post_meta( $post->ID, 'gutentor-font-fallback', true );
			$font_display    = get_post_meta( $post->ID, 'gutentor-font-display', true );
			$font_variations = get_post_meta( $post->ID, 'gutentor-font-variation', true );
			if ( ! $font_variations ) {
				$font_variations = array(
					array(
						'weight' => 400,
						'style'  => 'normal',
						'woff'   => '',
						'woff2'  => '',
						'ttf'    => '',
						'svg'    => '',
						'eot'    => '',
						'otf'    => '',
					),
				);
			}

			/*Security nonce*/
			wp_nonce_field( basename( __FILE__ ), 'gutentor_meta_nonce' );
			?>
			<div class="gutentor-font-callback components-base-control">
				<div class="components-base-control__field">
					<label class="components-base-control__label" for="gutentor-font-fallback">
						<h3><?php echo esc_html__( 'Font Fallback', 'gutentor' ); ?></h3>
						<p class="small-text">
							<?php esc_html_e( "Add the font's fallback names with comma(,) separator. eg. Arial, Helvetica, sans-serif", 'gutentor' ); ?>
						</p>
					</label>
					<input name="gutentor-font-fallback" value="<?php echo esc_attr( $font_fallback ); ?>" id="gutentor-font-fallback" class="widefat" />

				</div>
			</div>

			<div class="gutentor-font-callback components-base-control">
				<div class="components-base-control__field">
					<label class="components-base-control__label" for="gutentor-font-display">
						<h3><?php echo esc_html__( 'Font Display', 'gutentor' ); ?></h3>
					</label>
					<select id="gutentor-font-display" name="gutentor-font-display" class="widefat">
						<?php
						$options = array(
							array(
								'label' => esc_html__( 'Auto', 'gutentor' ),
								'value' => 'auto',
							),
							array(
								'label' => esc_html__( 'Block', 'gutentor' ),
								'value' => 'block',
							),
							array(
								'label' => esc_html__( 'Swap', 'gutentor' ),
								'value' => 'swap',
							),
							array(
								'label' => esc_html__( 'Fallback', 'gutentor' ),
								'value' => 'fallback',
							),
							array(
								'label' => esc_html__( 'Optional', 'gutentor' ),
								'value' => 'optional',
							),
						);
						foreach ( $options as $option ) {
							printf(
								'<option value="%1$s" %2$s>%3$s</option>',
								$option['value'],
								selected( $option['value'], $font_display, false ),
								$option['label']
							);
						}
						?>
					</select>
				</div>
			</div>

			<?php
			foreach ( $font_variations as $key => $font_variation ) {
				$font_variation = array_merge(
					array(
						'weight' => 400,
						'style'  => 'normal',
						'woff'   => '',
						'woff2'  => '',
						'ttf'    => '',
						'svg'    => '',
						'eot'    => '',
						'otf'    => '',
					),
					$font_variation
				);
				?>
				<div class="gutentor-fonts-variation components-base-control" id="gutentor-font-variation-<?php echo $key; ?>">
					<div class="gutentor-fonts-variation-header">
						<h3><?php esc_html_e( 'Font Variation' ); ?></h3>
						<div class="gutentor-fonts-variation-action">
							<button class="button button-primary button-large gutentor-fonts-variation-delete">
								<?php echo esc_html__( 'Delete', 'gutentor' ); ?>
							</button>
							<button class="button button-cancel button-large gutentor-fonts-variation-close">
								<span class="dashicons dashicons-arrow-down"></span>
								<span class="dashicons dashicons-arrow-up hidden"></span>
							</button>
						</div>
					</div>
					<div class="gutentor-fonts-variation-fields hidden">
						<div class="components-base-control__field">
							<?php
							$id   = 'gutentor-font-weight_' . $key;
							$name = 'gutentor-font-variation[' . $key . '][weight]';
							?>
							<label class="components-base-control__label" for="<?php echo esc_attr( $id ); ?>">
								<?php echo esc_html__( 'Font weight', 'gutentor' ); ?>
							</label>
							<select id="<?php echo esc_attr( $id ); ?>" name="<?php echo esc_attr( $name ); ?>" class="widefat">
								<?php
								$weight  = $font_variation['weight'];
								$options = array(
									array(
										'label' => 100,
										'value' => 100,
									),
									array(
										'label' => 200,
										'value' => 200,
									),
									array(
										'label' => 300,
										'value' => 300,
									),
									array(
										'label' => 400,
										'value' => 400,
									),
									array(
										'label' => 500,
										'value' => 500,
									),
									array(
										'label' => 600,
										'value' => 600,
									),
									array(
										'label' => 700,
										'value' => 700,
									),
									array(
										'label' => 800,
										'value' => 800,
									),
									array(
										'label' => 900,
										'value' => 900,
									),
								);
								foreach ( $options as $option ) {
									printf(
										'<option value="%1$s" %2$s>%3$s</option>',
										$option['value'],
										selected( $option['value'], $weight, false ),
										$option['label']
									);
								}
								?>
							</select>
						</div>
						<div class="components-base-control__field">
							<?php
							$id   = 'gutentor-font-style_' . $key;
							$name = 'gutentor-font-variation[' . $key . '][style]';
							?>
							<label class="components-base-control__label" for="<?php echo esc_attr( $id ); ?>">
								<?php echo esc_html__( 'Font style', 'gutentor' ); ?>
							</label>
							<select id="<?php echo esc_attr( $id ); ?>" name="<?php echo esc_attr( $name ); ?>" class="widefat">
								<?php
								$style   = $font_variation['style'];
								$options = array(
									array(
										'label' => esc_html__( 'Normal', 'gutentor' ),
										'value' => 'normal',
									),
									array(
										'label' => esc_html__( 'Italic', 'gutentor' ),
										'value' => 'italic',
									),
								);
								foreach ( $options as $option ) {
									printf(
										'<option value="%1$s" %2$s>%3$s</option>',
										$option['value'],
										selected( $option['value'], $style, false ),
										$option['label']
									);
								}
								?>
							</select>
						</div>
						<div class="components-base-control__field">
							<?php
							$id   = 'gutentor-font-woff_' . $key;
							$name = 'gutentor-font-variation[' . $key . '][woff]';
							?>
							<label class="components-base-control__label" for="<?php echo esc_attr( $id ); ?>">
								<?php echo esc_html__( 'File .woff', 'gutentor' ); ?>
							</label>
							<input type="url" name="<?php echo esc_attr( $name ); ?>" value="<?php echo esc_attr( $font_variation['woff'] ); ?>" id="<?php echo esc_attr( $id ); ?>" class="widefat" />
							<button class="button button-secondary gutentor-file-uploader-open" data-title="<?php esc_attr_e( 'File .woff', 'gutentor' ); ?>" data-button="<?php esc_attr_e( 'Add File .woff', 'gutentor' ); ?>">
								<?php echo esc_html__( 'Upload', 'gutentor' ); ?>
							</button>
						</div>
						<div class="components-base-control__field">
							<?php
							$id   = 'gutentor-font-woff2_' . $key;
							$name = 'gutentor-font-variation[' . $key . '][woff2]';
							?>
							<label class="components-base-control__label" for="<?php echo esc_attr( $id ); ?>">
								<?php echo esc_html__( 'File .woff2', 'gutentor' ); ?>
							</label>
							<input type="url" name="<?php echo esc_attr( $name ); ?>" value="<?php echo esc_attr( $font_variation['woff2'] ); ?>" id="<?php echo esc_attr( $id ); ?>" class="widefat" />
							<button class="button button-secondary gutentor-file-uploader-open" data-title="<?php esc_attr_e( 'File .woff2', 'gutentor' ); ?>" data-button="<?php esc_attr_e( 'Add File .woff2', 'gutentor' ); ?>">
								<?php echo esc_html__( 'Upload', 'gutentor' ); ?>
							</button>
						</div>
						<div class="components-base-control__field">
							<?php
							$id   = 'gutentor-font-ttf_' . $key;
							$name = 'gutentor-font-variation[' . $key . '][ttf]';
							?>
							<label class="components-base-control__label" for="<?php echo esc_attr( $id ); ?>">
								<?php echo esc_html__( 'File .ttf', 'gutentor' ); ?>
							</label>
							<input type="url" name="<?php echo esc_attr( $name ); ?>" value="<?php echo esc_attr( $font_variation['ttf'] ); ?>" id="<?php echo esc_attr( $id ); ?>" class="widefat" />
							<button class="button button-secondary gutentor-file-uploader-open" data-title="<?php esc_attr_e( 'File .ttf', 'gutentor' ); ?>" data-button="<?php esc_attr_e( 'Add File .ttf', 'gutentor' ); ?>">
								<?php echo esc_html__( 'Upload', 'gutentor' ); ?>
							</button>
						</div>
						<div class="components-base-control__field">
							<?php
							$id   = 'gutentor-font-svg_' . $key;
							$name = 'gutentor-font-variation[' . $key . '][svg]';
							?>
							<label class="components-base-control__label" for="<?php echo esc_attr( $id ); ?>">
								<?php echo esc_html__( 'File .svg', 'gutentor' ); ?>
							</label>
							<input type="url" name="<?php echo esc_attr( $name ); ?>" value="<?php echo esc_attr( $font_variation['svg'] ); ?>" id="<?php echo esc_attr( $id ); ?>" class="widefat" />
							<button class="button button-secondary gutentor-file-uploader-open" data-title="<?php esc_attr_e( 'File .svg', 'gutentor' ); ?>" data-button="<?php esc_attr_e( 'Add File .svg', 'gutentor' ); ?>">
								<?php echo esc_html__( 'Upload', 'gutentor' ); ?>
							</button>
						</div>
						<div class="components-base-control__field">
							<?php
							$id   = 'gutentor-font-eot_' . $key;
							$name = 'gutentor-font-variation[' . $key . '][eot]';
							?>
							<label class="components-base-control__label" for="<?php echo esc_attr( $id ); ?>">
								<?php echo esc_html__( 'File .eot', 'gutentor' ); ?>
							</label>
							<input type="url" name="<?php echo esc_attr( $name ); ?>" value="<?php echo esc_attr( $font_variation['eot'] ); ?>" id="<?php echo esc_attr( $id ); ?>" class="widefat" />
							<button class="button button-secondary gutentor-file-uploader-open" data-title="<?php esc_attr_e( 'File .eot', 'gutentor' ); ?>" data-button="<?php esc_attr_e( 'Add File .eot', 'gutentor' ); ?>">
								<?php echo esc_html__( 'Upload', 'gutentor' ); ?>
							</button>
						</div>
						<div class="components-base-control__field">
							<?php
							$id   = 'gutentor-font-otf_' . $key;
							$name = 'gutentor-font-variation[' . $key . '][otf]';
							?>
							<label class="components-base-control__label" for="<?php echo esc_attr( $id ); ?>">
								<?php echo esc_html__( 'File .otf', 'gutentor' ); ?>
							</label>
							<input type="url" name="<?php echo esc_attr( $name ); ?>" value="<?php echo esc_attr( $font_variation['otf'] ); ?>" id="<?php echo esc_attr( $id ); ?>" class="widefat" />
							<button class="button button-secondary gutentor-file-uploader-open" data-title="<?php esc_attr_e( 'File .otf', 'gutentor' ); ?>" data-button="<?php esc_attr_e( 'Add File .otf', 'gutentor' ); ?>">
								<?php echo esc_html__( 'Upload', 'gutentor' ); ?>
							</button>
						</div>
					</div>
				</div>
				<?php
			}
			?>
			<div id="gutentor-append-repeater"></div>
			<input class="gutentor-total-font-variation" type="hidden" value="<?php echo count( $font_variations ); ?>">
			<span class="button-primary gutentor-add-font-variation">
				<i class="dashicons dashicons-plus"></i>
				<?php echo esc_html__( 'Add Font Variation', 'gutentor' ); ?>
			</span>
			<script type="text/javascript">
				jQuery(document).ready(function ($){
					$('.gutentor-add-font-variation').click(function (e){
						e.preventDefault();

						let totalRepeater = $('.gutentor-total-font-variation').val(),
							item = $('.gutentor-fonts-variation').first().clone().prop('id', 'gutentor-font-variation-' + ( parseInt( totalRepeater ) ) );
						item.appendTo('#gutentor-append-repeater');

						let newItem = $('#gutentor-font-variation-' + totalRepeater);
						console.log(newItem.find('select').attr('id'))
						newItem.find('label').attr('for',newItem.find('label').attr('for').replace(/[0-9]+/,totalRepeater));
						newItem.find('select').attr('id',newItem.find('select').attr('id').replace(/[0-9]+/,totalRepeater));
						newItem.find('select').attr('name',newItem.find('select').attr('name').replace(/[0-9]+/,totalRepeater));
						newItem.find('input').attr('id',newItem.find('input').attr('id').replace(/[0-9]+/,totalRepeater));
						newItem.find('input').attr('name',newItem.find('input').attr('name').replace(/[0-9]+/,totalRepeater));

						newItem.find('input, select').val('');

						$('.gutentor-total-font-variation').val(parseInt( totalRepeater )+1)
					});

					/*File loader*/
					let FPFILEUPLOAD = function () {
						/*File Meta*/
						$( document ).on('click', '.gutentor-file-uploader-open', function(e){
								// Prevents the default action from occuring.
								e.preventDefault();

								let media_title = $( this ).data( 'title' ),
									media_button    = $( this ).data( 'button' ),
									media_input_val = $( this ).prev();

								meta_image_frame = wp.media.frames.meta_image_frame = wp.media(
									{
										title: media_title,
										button: { text:  media_button },
									}
								);
								// Opens the media library frame.
								meta_image_frame.open();

								// Runs when an File is selected.
								meta_image_frame.on(
									'select',
									function(){
										// Grabs the attachment selection and creates a JSON representation of the model.
										var media_attachment = meta_image_frame.state().get( 'selection' ).first().toJSON();

										// Sends the attachment ID/URL to our custom File input field.
										media_input_val.val( media_attachment.url );
										media_input_val.trigger( 'change' );
									}
								);
							}
						);
					};
					FPFILEUPLOAD();

					/*Delete Repeater*/
					$( document ).on('click', '.gutentor-fonts-variation-delete', function(e){
						e.preventDefault();
						let totalRepeater = $('.gutentor-total-font-variation').val();
						if( totalRepeater > 1 ){
							$( this ).closest( '.gutentor-fonts-variation' ).remove();
							$('.gutentor-total-font-variation').val(parseInt( totalRepeater )-1);
						}
					});

					/*Close Repeater*/
					$( document ).on('click', '.gutentor-fonts-variation-close', function(e){
						e.preventDefault();
						$('.dashicons-arrow-down,.dashicons-arrow-up').toggleClass('hidden');
						$( this ).closest( '.gutentor-fonts-variation' ).find('.gutentor-fonts-variation-fields').slideToggle();
					});
				})
			</script>
			<?php
		}

		/**
		 * Handles saving the meta box.
		 *
		 * @param int     $post_id Post ID.
		 * @param WP_Post $post    Post object.
		 * @return null
		 */
		public function save_metabox( $post_id, $post ) {
			/*
			  * A Guide to Writing Secure Themes – Part 4: Securing Post Meta
			  *https://make.wordpress.org/themes/2015/06/09/a-guide-to-writing-secure-themes-part-4-securing-post-meta/
			  * */
			if (
				! isset( $_POST['gutentor_meta_nonce'] ) ||
				! wp_verify_nonce( $_POST['gutentor_meta_nonce'], basename( __FILE__ ) ) || /*Protecting against unwanted requests*/
				( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) || /*Dealing with autosaves*/
				! current_user_can( 'edit_post', $post_id )/*Verifying access rights*/
			) {
				return false;
			}

			if ( $this->post_type !== $_POST['post_type'] ) {
				return $post_id;
			}

			if ( ! current_user_can( 'edit_post', $post_id ) ) {
				return $post_id;
			}

			// Execute this saving function
			// Font Fallback
			if ( isset( $_POST['gutentor-font-fallback'] ) ) {
				$old = get_post_meta( $post_id, 'gutentor-font-fallback', true );
				$new = sanitize_text_field( $_POST['gutentor-font-fallback'] );
				if ( $new && $new != $old ) {
					update_post_meta( $post_id, 'gutentor-font-fallback', $new );
				} elseif ( '' == $new && $old ) {
					delete_post_meta( $post_id, 'gutentor-font-fallback', $old );
				}
			}
			/*Font display*/
			if ( isset( $_POST['gutentor-font-display'] ) ) {
				$old = get_post_meta( $post_id, 'gutentor-font-display', true );
				$new = sanitize_text_field( $_POST['gutentor-font-display'] );
				if ( $new && $new != $old ) {
					update_post_meta( $post_id, 'gutentor-font-display', $new );
				} elseif ( '' == $new && $old ) {
					delete_post_meta( $post_id, 'gutentor-font-display', $old );
				}
			}

			// Font Variation
			if ( isset( $_POST['gutentor-font-variation'] ) ) {
				$old = get_post_meta( $post_id, 'gutentor-font-variation', true );
				$new = array();
				if ( is_array( $_POST['gutentor-font-variation'] ) ) {
					foreach ( $_POST['gutentor-font-variation'] as $key => $font_variation ) {
						foreach ( $font_variation as $key1 => $item_value ) {
							$key                  = sanitize_key( $key );
							$key1                 = sanitize_key( $key1 );
							$new[ $key ][ $key1 ] = sanitize_text_field( $item_value );
						}
					}
				}
				if ( $new && $new != $old ) {
					update_post_meta( $post_id, 'gutentor-font-variation', $new );
				} elseif ( '' == $new && $old ) {
					delete_post_meta( $post_id, 'gutentor-font-variation', $old );
				}
			}

			$this->set_dynamic_fonts_transists();
		}
	}
}

/**
 * Begins execution of the hooks.
 *
 * @since    1.0.0
 */
function gutentor_fonts() {
	return Gutentor_Fonts::instance();
}
gutentor_fonts()->run();
