<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Gutentor_Dynamic_Content' ) ) :

	/**
	 * Create Dynamic CSS
	 *
	 * @package Gutentor
	 * @since 1.0.0
	 */
	class Gutentor_Dynamic_Content {

		/**
		 * Rest route namespace.
		 *
		 * @var Gutentor_Self_Api_Handler
		 */
		public $namespace = 'gutentor-self-api/';

		/**
		 * Rest route version.
		 *
		 * @var Gutentor_Self_Api_Handler
		 */
		public $version = 'v1';

		/**
		 * Main Instance
		 *
		 * Insures that only one instance of Gutentor_Dynamic_Content exists in memory at any one
		 * time. Also prevents needing to define globals all over the place.
		 *
		 * @since    1.0.0
		 * @access   public
		 *
		 * @return object
		 */
		public static function instance() {

			// Store the instance locally to avoid private static replication
			static $instance = null;

			// Only run these methods if they haven't been ran previously
			if ( null === $instance ) {
				$instance = new Gutentor_Dynamic_Content();
			}

			// Always return the instance
			return $instance;
		}

		/**
		 * Run functionality with hooks
		 *
		 * @since    1.0.0
		 * @access   public
		 *
		 * @return void
		 */
		public function run() {
			add_action( 'render_block', array( $this, 'render_dc' ), 99999, 2 );
			add_action( 'rest_api_init', array( $this, 'register_routes' ) );

		}

		/**
		 * Register REST API route
		 */
		public function register_routes() {
			$namespace = $this->namespace . $this->version;

			register_rest_route(
				$namespace,
				'/get_backend_dc',
				array(
					array(
						'methods'             => \WP_REST_Server::READABLE,
						'callback'            => array( $this, 'get_backend_dc' ),
						'permission_callback' => function () {
							return current_user_can( 'edit_posts' );
						},
					),
				)
			);
		}

		/**
		 * Function to fetch backend dynamic content.
		 */
		public function get_backend_dc( \WP_REST_Request $request ) {

			$raw_args = $request->get_param( 'args' );
			$raw_args = $this->get_dc( $raw_args );
			return rest_ensure_response( $raw_args );
		}

		/**
		 * Get Field Default Value
		 * get_field_default_value
		 *
		 * @since    3.1.1
		 * @access   public
		 *
		 * @param string $src
		 * @return string
		 */
		public function get_field_default_value( $src ) {

			switch ( $src ) {
				case 'postType':
				case 'currentPost':
					$dc_default_output = 'title';
					break;
				case 'site':
					$dc_default_output = 'title';
					break;
				case 'author':
					$dc_default_output = 'name';
					break;
				case 'taxonomy':
					$dc_default_output = 'title';
					break;
				default:
					$dc_default_output = '';
					break;
			}
			return $dc_default_output;

		}

		/**
		 * Parse Args for query
		 *
		 * @since    1.0.0
		 * @access   public
		 * @param $pre_args string URL
		 *
		 * @return array
		 */
		public function parse_args( $pre_args ) {
			$dc_fields = parse_url( $pre_args );
			$args      = array();
			if ( isset( $dc_fields['query'] ) ) {
				parse_str( html_entity_decode( $dc_fields['query'] ), $query );
				$args = $query;
			}
			if ( array_key_exists( 'source', $args ) ) {
				if ( $args['source'] === 'postType' && ! array_key_exists( 'postType', $args ) ) {
					$args['postType'] = 'post';
				}
				if ( ! array_key_exists( 'field', $args ) ) {
					$args['field'] = $this->get_field_default_value( $args['source'] );
				}
			}
			return $args;
		}

		/**
		 * Get Post Dynamic Content
		 * get_post_dc
		 *
		 * @since    3.1.1
		 * @access   public
		 *
		 * @param array $args
		 * @return string
		 */
		public function get_post_dc( $args ) {
			$dc_post_output = '';
			$dc_args        = ! empty( $args ) ? $args : false;
			if ( ! $dc_args ) {
				return $dc_post_output;
			}
			$post_type   = is_array( $dc_args ) && array_key_exists( 'postType', $dc_args ) ? $dc_args['postType'] : false;
			$post_id     = is_array( $dc_args ) && array_key_exists( 'value', $dc_args ) ? $dc_args['value'] : false;
			$post_field  = is_array( $dc_args ) && array_key_exists( 'field', $dc_args ) ? $dc_args['field'] : false;
			$post_object = get_post( $post_id );

			/*
			TODO for Current Post need Post inside query block
			https://github.com/WordPress/gutenberg/issues/35676*/
			/*
			if ( isset( $args['isBackend'] ) && isset($args['isCurrentPost'] )) {
				if( 'fImage' === $post_field ){
					$post_field = 'Featured Image';
				}
				return 'Current Post '.$post_field;

			}*/
			switch ( $post_field ) {
				case 'ID':
					$dc_post_output = $post_id;
					break;
				case 'title':
					$dc_post_output = get_the_title( $post_id );
					break;
				case 'url':
					$dc_post_output = get_the_permalink( $post_id );
					break;
				case 'fImage':
					if ( has_post_thumbnail( $post_id ) ) {
						$dc_post_output = get_the_post_thumbnail_url( $post_id, 'full' );
					} else {
						$dc_post_output = GUTENTOR_URL . 'assets/img/default-image.jpg';
					}
					break;
				case 'slug':
					$dc_post_output = basename( get_the_permalink( $post_id ) );
					break;
				case 'excerpt':
					$dc_post_output = is_object( $post_object ) ? $post_object->post_excerpt : '';
					break;
				case 'content':
					$dc_post_output = is_object( $post_object ) ? $post_object->post_content : '';
					break;
				case 'date':
					$dc_post_output = get_the_time( 'Y-m-d', $post_id );
					break;
				case 'dateGMT':
					$dc_post_output = is_object( $post_object ) ? $post_object->post_date_gmt : '';
					break;
				case 'modifiedDate':
					$dc_post_output = get_the_modified_date( 'Y-m-d', $post_id );
					break;
				case 'modifiedGMT':
					$dc_post_output = is_object( $post_object ) ? $post_object->post_modified_gmt : '';
					break;
				case 'postType':
					$dc_post_output = is_object( $post_object ) ? $post_object->post_type : '';
					break;
				case 'status':
					$dc_post_output = is_object( $post_object ) ? $post_object->post_status : '';
					break;
				case 'cField':
					$post_meta_key  = is_array( $dc_args ) && array_key_exists( 'metaKey', $dc_args ) ? $dc_args['metaKey'] : false;
					$dc_post_output = get_post_meta( $post_id, $post_meta_key, true );
					break;
				default:
					$dc_post_output = '';
					break;
			}
			return $dc_post_output;
		}

		/**
		 * Get Site Dynamic Content
		 * get_site_dc
		 *
		 * @since    3.1.1
		 * @access   public
		 *
		 * @param array $args
		 * @return string
		 */
		public function get_site_dc( $args ) {
			$dc_site_output = '';
			$dc_args        = ! empty( $args ) ? $args : false;
			if ( ! $dc_args ) {
				return $dc_site_output;
			}
			$post_field = is_array( $dc_args ) && array_key_exists( 'field', $dc_args ) ? $dc_args['field'] : false;
			switch ( $post_field ) {
				case 'title':
					$dc_site_output = get_bloginfo( 'name' );
					break;
				case 'tagline':
					$dc_site_output = get_bloginfo( 'description' );
					break;
				case 'url':
					$dc_site_output = get_site_url();
					break;
				case 'logo':
					$dc_site_output = esc_url( wp_get_attachment_url( get_theme_mod( 'custom_logo' ) ) );
					break;
				default:
					$dc_site_output = '';
					break;
			}
			return $dc_site_output;
		}

		/**
		 * Get Author Dynamic Content
		 * get_author_dc
		 *
		 * @since    3.1.1
		 * @access   public
		 *
		 * @param array $args
		 * @return mixed
		 */
		public function get_author_dc( $args ) {
			$dc_author_output = '';
			$dc_args          = ! empty( $args ) ? $args : false;

			if ( ! $dc_args ) {
				return $dc_author_output;
			}
			$author_id    = is_array( $dc_args ) && array_key_exists( 'author', $dc_args ) ? $dc_args['author'] : false;
			$author_field = is_array( $dc_args ) && array_key_exists( 'field', $dc_args ) ? $dc_args['field'] : false;
			$author_data  = get_userdata( $author_id );
			switch ( $author_field ) {
				case 'ID':
					$dc_author_output = $author_id;
					break;
				case 'name':
					$dc_author_output = is_object( $author_data ) ? $author_data->display_name : '';
					break;
				case 'postsUrl':
					$dc_author_output = get_author_posts_url( $author_id );
					break;
				case 'profileImg':
					$dc_author_output = get_avatar_url( $author_id );
					break;
				case 'postsCount':
					$dc_author_output = count_user_posts( $author_id );
					break;
				case 'fName':
					$dc_author_output = is_object( $author_data ) ? $author_data->first_name : '';
					break;
				case 'lName':
					$dc_author_output = is_object( $author_data ) ? $author_data->last_name : '';
					break;
				case 'desc':
					$dc_author_output = is_object( $author_data ) ? $author_data->description : '';
					break;
				default:
					$dc_author_output = '';
					break;
			}
			return $dc_author_output;

		}

		/**
		 * Get Term Dynamic Content
		 * get_term_dc
		 *
		 * @since    1.0.0
		 * @access   public
		 *
		 * @param array $args
		 * @return string
		 */
		public function get_term_dc( $args ) {
			$dc_term_output = '';
			$dc_args        = ! empty( $args ) ? $args : false;
			if ( ! $dc_args ) {
				return $dc_term_output;
			}
			$taxonomy   = is_array( $dc_args ) && array_key_exists( 'taxonomy', $dc_args ) ? $dc_args['taxonomy'] : false;
			$term_id    = is_array( $dc_args ) && array_key_exists( 'value', $dc_args ) ? $dc_args['value'] : false;
			$term_field = is_array( $dc_args ) && array_key_exists( 'field', $dc_args ) ? $dc_args['field'] : false;
			$term_data  = get_term( $term_id, $taxonomy );
			switch ( $term_field ) {
				case 'ID':
					$dc_term_output = $term_id;
					break;
				case 'title':
					$dc_term_output = is_object( $term_data ) ? $term_data->name : '';
					break;
				case 'description':
					$dc_term_output = is_object( $term_data ) ? $term_data->description : '';
					break;
				case 'link':
					$dc_term_output = is_object( $term_data ) ? get_term_link( $term_data ) : '';
					break;
				case 'img':
					$image_url = wp_get_attachment_image_src( Gutentor_Duplex_P6_T1::get_instance()->get_term_thumbnail_id( $term_data ), 'full' );
					if ( ! $image_url ) {
						$image_url[0] = GUTENTOR_URL . 'assets/img/default-image.jpg';
					}
					$dc_term_output = $image_url[0];
					break;
				default:
					$dc_term_output = '';
					break;
			}
			return $dc_term_output;
		}


		/**
		 * Get Dynamic Content
		 *
		 * @return string
		 */
		public function get_dc( $args ) {
			$query_args = $this->parse_args( $args );
			$dc         = 'Default';
			if ( ! $query_args ) {
				return $dc;
			}
			switch ( $query_args['source'] ) {
				case 'postType':
					$dc = $this->get_post_dc( $query_args );
					break;
				case 'currentPost':
					if ( isset( $query_args['isBackend'] ) ) {
						$query_args['isCurrentPost'] = true;
					} else {
						$query_args['value'] = get_the_ID();
					}

					$dc = $this->get_post_dc( $query_args );
					break;
				case 'site':
					$dc = $this->get_site_dc( $query_args );
					break;
				case 'author':
					$dc = $this->get_author_dc( $query_args );
					break;
				case 'taxonomy':
					$dc = $this->get_term_dc( $query_args );
					break;

			}
			return $dc;
		}

		/**
		 * Replace Link
		 *
		 * @return string
		 */
		public function replace_link( $block_content, $pattern ) {
			preg_match_all( $pattern, $block_content, $matches );

			if ( ! is_array( $matches ) || ! isset( $matches[1] ) || ! is_array( $matches[1] ) ) {
				return $block_content;
			}

			foreach ( $matches[1] as $dc_todo_replace ) {
				if ( strpos( $dc_todo_replace, 'data-g-dcl' ) !== false ) {

					$output        = gutentor_dynamic_content()->get_dc( $dc_todo_replace );
					$block_content = str_replace( $dc_todo_replace, $output, $block_content );
				}
			}
			return $block_content;
		}

		/**
		 * Render Dynamic Field
		 *
		 * @return string
		 */
		public function dc_field( $block_content ) {

			$pattern = '/<span data-g-dc="(.*?(?="))"[^>]*>(.*?)<\/span[^\>]*>/';
			preg_match_all( $pattern, $block_content, $matches );

			if ( ! is_array( $matches[0] ) ) {
				return $block_content;
			}
			foreach ( $matches[0] as $dc_todo_replace ) {
				preg_match( '/data-g-dc="(.*?(?="))"/', $dc_todo_replace, $pre_args );
				$output        = gutentor_dynamic_content()->get_dc( $pre_args[1] );
				$block_content = str_replace( $dc_todo_replace, $output, $block_content );
			}
			return $block_content;
		}

		/**
		 * Render Dynamic Link
		 * Reference
         * https://www.sitepoint.com/community/t/find-urls-in-a-string-with-preg-match-all/6826/4
		 * @return string
		 */
		public function dc_link( $block_content ) {
			// $pattern = '/<a [^>]* href="(.*?[^>]*data-g-dcl[^>]*?)"[^>]*>(.*?)<\/a[^\>]*>/';

			$href_regex  = '<'; // 1 start of the tag
			$href_regex .= '\\s*'; // 2 zero or more whitespace
			$href_regex .= 'a'; // 3 the a of the tag itself
			$href_regex .= '\\s+'; // 4 one or more whitespace
			$href_regex .= '[^>]*'; // 5 zero or more of any character that is _not_ the end of the tag
			$href_regex .= 'href'; // 6 the href bit of the tag
			$href_regex .= '\\s*'; // 7 zero or more whitespace
			$href_regex .= '='; // 8 the = of the tag
			$href_regex .= '\\s*'; // 9 zero or more whitespace
			$href_regex .= '[\\"' . "']?"; // 10 none or one of " or '
			$href_regex .= '('; // 11 opening parenthesis, start of the bit we want to capture
			$href_regex .= '[^>]*'; // 12 zero or more of any character that is _not_ the end of the tag
			$href_regex .= 'data-g-dcl'; // 13 keyword on the link that must present
			$href_regex .= '[^>]*'; // 14 zero or more of any character that is _not_ the end of the tag

			$href_regex .= '[^\\"' . "' >]+"; // 15 one or more of any character _except_ our closing characters
			$href_regex .= ')'; // 16 closing parenthesis, end of the bit we want to capture
			$href_regex .= '[\\"' . "' >]"; // 17 closing chartacters of the bit we want to capture

			$regex  = '/'; // regex start delimiter
			$regex .= $href_regex;
			$regex .= '/'; // regex end delimiter
			$regex .= 'i'; // Pattern Modifier - makes regex case insensative
			$regex .= 's'; // Pattern Modifier - makes a dot metacharater in the pattern
			// match all characters, including newlines
//			$regex .= 'U'; // Pattern Modifier - makes the regex ungready

			$pattern = $regex;

			/*issue is here in patter filter*/
			$block_content = $this->replace_link( $block_content, $pattern );
			return $block_content;
		}

		/**
		 * Render Dynamic Link
		 *
		 * @return string
		 */
		public function dc_image( $block_content ) {
			$pattern       = '/<img [^>]* src="(.*?[^>]*data-g-dcimg[^>]*)"[^>]*>/';
			$block_content = $this->replace_link( $block_content, $pattern );
			return $block_content;
		}

		/**
		 * Render Dynamic Content
		 *
		 * @return string
		 */
		public function render_dc( $block_content, $block ) {

			if ( is_admin() ) {
				return $block_content;
			}

			/*Link*/
			if ( strpos( $block_content, 'data-g-dcl' ) !== false ) {
				$block_content = $this->dc_link( $block_content );
			}

			/*image*/
			if ( strpos( $block_content, 'data-g-dcimg' ) !== false ) {
				$block_content = $this->dc_image( $block_content );
			}

			/*field*/
			if ( strpos( $block_content, 'data-g-dc=' ) !== false ) {
				$block_content = $this->dc_field( $block_content );
			}
			return $block_content;

		}


	}
endif;

/**
 * Call Gutentor_Dynamic_Content
 *
 * @since    1.0.0
 * @access   public
 */
if ( ! function_exists( 'gutentor_dynamic_content' ) ) {

	function gutentor_dynamic_content() {

		return Gutentor_Dynamic_Content::instance();
	}
	gutentor_dynamic_content()->run();
}
