<?php
/**
 * Gutentor Custom Popup
 *
 * @since 1.0.0
 */

if ( ! class_exists( 'Gutentor_Template' ) ) {

	/**
	 * Class Gutentor_Template.
	 */
	class Gutentor_Template {

		/**
		 *  Post Type.
		 *
		 * @var int $paged
		 */
		public $post_type = 'gutentor-template';

		/**
		 * Main Gutentor_Template Instance
		 * Initialize the class and set its properties.
		 *
		 * @return object $instance Gutentor_Template Instance
		 * @since    1.0.0
		 */
		public static function instance() {
			// Store the instance locally to avoid private static replication
			static $instance = null;

			// Only run these methods if they haven't been ran previously
			if ( null === $instance ) {
				$instance = new Gutentor_Template();
			}

			// Always return the instance
			return $instance;
		}

		public function run() {
			if ( ! gutentor_get_options( 'on-popup' ) ) {
				return;
			}

			add_action( 'init', array( $this, 'register_template' ), 10 );
			add_action( 'init', array( $this, 'register_popup_meta' ), 20 );
			add_action( 'admin_menu', array( $this, 'add_post_type_submenu' ) );

			add_filter( 'single_template', array( $this, 'popup_template' ), 999 );
			add_filter( 'wp_enqueue_scripts', array( $this, 'popup_scripts' ), 999 );
			add_filter( 'gutentor_block_frontend_localize_data', array( $this, 'add_localize_data' ), 999 );

			// Add the custom columns to the gutentor-template post type:
			add_filter( 'manage_gutentor-template_posts_columns', array( $this, 'set_custom_edit_gctemplate_columns' ) );

			// Add the data to the custom columns for the gutentor-template post type:
			add_action( 'manage_gutentor-template_posts_custom_column', array( $this, 'custom_gctemplate_column' ), 10, 2 );

			add_action( 'save_post', array( $this, 'update_template_taxonomies' ), 10 );

			/*Add Popup HTML on footer*/
			add_action( 'wp_footer', array( $this, 'add_footer_popup' ), -1 );
		}

		/**
		 * @param $columns array.
		 *
		 * @since 1.0.0
		 * @access public
		 *
		 * @return mixed
		 */
		public function set_custom_edit_gctemplate_columns( $columns ) {
			$columns['type'] = __( 'Type', 'gutentor' );
			return $columns;
		}

		/**
		 * @param $column array.
		 * @param $post_id int.
		 *
		 * @since 1.0.0
		 * @access public
		 *
		 * @return mixed
		 */
		function custom_gctemplate_column( $column, $post_id ) {
			switch ( $column ) {

				case 'type':
					$terms = get_the_terms( $post_id, 'gutentor_template_taxonomies' );
					if ( $terms && ! is_wp_error( $terms ) ) :
						$entry_terms = '';
						foreach ( $terms as $term ) {
							$entry_terms .= $term->name . ', ';
						}
						$entry_terms = rtrim( $entry_terms, ', ' );
						if ( ! gutentor_get_options( 'on-popup' ) ) {
							$entry_terms .= ' ' . esc_html__( '(Popup is disabled from Gutentor setting)', 'gutentor' );
						}
						echo $entry_terms;
					else :
						esc_html_e( 'No terms found', 'gutentor' );
					endif;

			}
		}

		function update_template_taxonomies( $post_id ) {

			if ( 'gutentor-template' !== get_post_type( $post_id ) ) {
					return $post_id;
			}
			if ( ! has_term( '', 'gutentor_template_taxonomies', $post_id ) ) {
				// Assign a term to our post
				wp_set_post_terms( $post_id, 'popup', 'gutentor_template_taxonomies' );
			}
		}

		/**
		 * Register the register_post_type.
		 *
		 * @since    1.0.0
		 */
		public function add_post_type_submenu() {
			add_submenu_page(
				'gutentor',
				_x( 'Templates', 'Post type general name', 'gutentor' ),
				_x( 'Template', 'Post type general name', 'gutentor' ),
				'manage_options',
				'edit.php?post_type=gutentor-template'
			);
		}

		/**
		 * Initialization Custom Popup
		 * *
		 *
		 * @since 1.0.0
		 * @access public
		 *
		 * @see __construct
		 */
		public function register_template() {
			$labels = apply_filters(
				'gutentor_post_type_popup_labels',
				array(
					'name'                  => _x( 'Templates', 'post type general name' ),
					'singular_name'         => _x( 'Template', 'post type singular name' ),
					'add_new'               => _x( 'Add New', 'Template' ),
					'add_new_item'          => __( 'Add New Template' ),
					'new_item'              => __( 'New Template' ),
					'edit_item'             => __( 'Edit Template' ),
					'view_item'             => __( 'View Template' ),
					'all_items'             => __( 'Templates' ),
					'search_items'          => __( 'Search Templates' ),
					'parent_item_colon'     => __( 'Parent Template:' ),
					'not_found'             => __( 'No templates found.' ),
					'not_found_in_trash'    => __( 'No templates found in Trash.' ),
					'archives'              => __( 'Template archives' ),
					'insert_into_item'      => __( 'Insert into template' ),
					'uploaded_to_this_item' => __( 'Uploaded to this template' ),
					'filter_items_list'     => __( 'Filter templates list' ),
					'items_list_navigation' => __( 'Templates list navigation' ),
					'items_list'            => __( 'Templates list' ),
				)
			);

			$args = array(
				'labels'             => $labels,
				'description'        => __( 'Templates to include in your theme.' ),
				'public'             => true,
				'publicly_queryable' => true,
				/*
				update it after completing popup functionality*/
				// 'show_ui'               => wp_is_block_theme(),
				'show_in_menu'       => false,
				'query_var'          => true,
				'show_in_rest'       => true,
				'rewrite'            => false,
				'capability_type'    => 'post',
				'has_archive'        => false,
				'hierarchical'       => false,
				'menu_position'      => null,
				'supports'           => array( 'title', 'editor', 'author', 'custom-fields' ),

			);

			register_post_type( 'gutentor-template', $args );

			register_taxonomy(
				'gutentor_template_taxonomies',
				array( 'gutentor-template' ),
				array(
					'public'            => false,
					'hierarchical'      => false,
					'labels'            => array(
						'name'          => __( 'Popup' ),
						'singular_name' => __( 'Popup' ),
					),
					'query_var'         => false,
					'rewrite'           => false,
					'show_ui'           => false,
					'show_in_nav_menus' => false,
					'show_in_rest'      => false,
				)
			);

			wp_insert_term(
				'Popup',
				'gutentor_template_taxonomies',
				array(
					'slug' => 'popup',
				)
			);
		} //end init

		/**
		 * Initialization Custom Popup
		 * *
		 *
		 * @since 1.0.0
		 * @access public
		 *
		 * @see __construct
		 */
		public function register_popup_meta() {
			/*adding/updating/deleting attrs will reset the value of the meta e.g gutentor_p_type = null*/
			register_post_meta(
				'gutentor-template',
				'gutentor_p_type',
				array(
					'single'       => true,
					'type'         => 'object',
					'show_in_rest' => array(
						'schema' => array(
							'type'       => 'object',
							'properties' => array(
								'info'      => array(
									'type' => 'boolean',
								),
								'type'      => array(
									'type' => 'string',
								),
								'side'      => array(
									'type' => 'string',
								),
								'sideUnit'  => array(
									'type' => 'string',
								),
								'x'         => array(
									'type' => 'number',
								),
								'y'         => array(
									'type' => 'number',
								),
								'widthUnit' => array(
									'type' => 'string',
								),
								'widthVal'  => array(
									'type' => 'number',
								),
							),
						),
					),
				)
			);

			/*adding/updating/deleting attrs will reset the value of the meta e.g gutentor_p_condition = null*/
			register_post_meta(
				'gutentor-template',
				'gutentor_p_condition',
				array(
					'single'       => true,
					'type'         => 'array',
					'show_in_rest' => array(
						'schema' => array(
							'items' => array(
								'type'       => 'object',
								'properties' => array(
									'ie'       => array(
										'type' => 'string',
									),
									'scope'    => array(
										'type' => 'string',
									),
									'post'     => array(
										'type'   => 'array',
										'schema' => array(
											'items' => array(
												'type' => 'object',
												'properties' => array(
													'label' => array(
														'type' => 'string',
													),
													'value' => array(
														'type' => 'string',
													),
												),
											),
										),
									),
									'tax'      => array(
										'type'   => 'array',
										'schema' => array(
											'items' => array(
												'type' => 'object',
												'properties' => array(
													'label' => array(
														'type' => 'string',
													),
													'value' => array(
														'type' => 'string',
													),
												),
											),
										),
									),
									'author'   => array(
										'type'   => 'array',
										'schema' => array(
											'items' => array(
												'type' => 'object',
												'properties' => array(
													'label' => array(
														'type' => 'string',
													),
													'value' => array(
														'type' => 'string',
													),
												),
											),
										),
									),
									'singular' => array(
										'type'   => 'array',
										'schema' => array(
											'items' => array(
												'type' => 'object',
												'properties' => array(
													'label' => array(
														'type' => 'string',
													),
													'value' => array(
														'type' => 'string',
													),
												),
											),
										),
									),
									'id'       => array(
										'type' => 'string',
									),
								),
							),
						),
					),
				)
			);

			/*adding/updating/deleting attrs will reset the value of the meta e.g gutentor_p_trigger = null*/
			register_post_meta(
				'gutentor-template',
				'gutentor_p_trigger',
				array(
					'single'       => true,
					'type'         => 'object',
					'show_in_rest' => array(
						'schema' => array(
							'type'       => 'object',
							'properties' => array(
								'trigger'    => array(
									'type' => 'string',
								),
								'timing'     => array(
									'type' => 'string',
								),
								'direction'  => array(
									'type' => 'string',
								),
								'scrollUnit' => array(
									'type' => 'string',
								),
								'scrollVal'  => array(
									'type' => 'number',
								),
								'class'      => array(
									'type' => 'string',
								),
							),
						),
					),
				)
			);

			/*adding/updating/deleting attrs will reset the value of the meta e.g gutentor_p_show_device = null*/
			register_post_meta(
				'gutentor-template',
				'gutentor_p_show_device',
				array(
					'type'         => 'array',
					'show_in_rest' => array(
						'schema' => array(
							'items' => array(
								'type'       => 'object',
								'properties' => array(
									'label' => array(
										'type' => 'string',
									),
									'value' => array(
										'type' => 'string',
									),
								),
							),
						),
					),
					'single'       => true,
				)
			);

			/*adding/updating/deleting attrs will reset the value of the meta e.g gutentor_p_options = null*/
			register_post_meta(
				'gutentor-template',
				'gutentor_p_options',
				array(
					'single'       => true,
					'type'         => 'object',
					'show_in_rest' => array(
						'schema' => array(
							'type'       => 'object',
							'properties' => array(
								'animation'       => array(
									'type' => 'string',
								),
								'closeOnBgClick'  => array(
									'type' => 'boolean',
								),
								'closeBtnInside'  => array(
									'type' => 'boolean',
								),
								'showCloseBtn'    => array(
									'type' => 'boolean',
								),
								'enableEscapeKey' => array(
									'type' => 'boolean',
								),
								'fixedContentPos' => array(
									'type' => 'boolean',
								),
								'fixedBgPos'      => array(
									'type' => 'boolean',
								),
								'closeBtnCol'     => array(
									'type' => 'string',
								),
								'bgType'          => array(
									'type' => 'string',
								),
								'bgCol'           => array(
									'type' => 'string',
								),
								'bgGradient'      => array(
									'type' => 'string',
								),
							),
						),
					),
				)
			);
		} //end init

		public function popup_scripts() {
			/*Maginific popup*/
			wp_enqueue_style( 'magnific-popup' );
			wp_style_add_data( 'magnific-popup', 'rtl', 'replace' );
			// magnify popup  Js
			wp_enqueue_script( 'magnific-popup' );
		}

		/*
		Source
		https://gist.github.com/Auke1810/f2a4cf04f2c07c74a393a4b442f22267
		*/
		public function popup_template( $template ) {
			global $post;

			if ( $this->post_type === $post->post_type && locate_template( array( 'single-gutentor-template.php' ) ) !== $template ) {

				remove_action( 'wp_head', 'rsd_link' ); // remove really simple discovery link
				remove_action( 'wp_head', 'wp_generator' ); // remove WordPress version

				remove_action( 'wp_head', 'feed_links', 2 ); // remove rss feed links (make sure you add them in yourself if youre using feedblitz or an rss service)
				remove_action( 'wp_head', 'feed_links_extra', 3 ); // removes all extra rss feed links

				remove_action( 'wp_head', 'index_rel_link' ); // remove link to index page
				remove_action( 'wp_head', 'wlwmanifest_link' ); // remove wlwmanifest.xml (needed to support windows live writer)

				remove_action( 'wp_head', 'start_post_rel_link', 10, 0 ); // remove random post link
				remove_action( 'wp_head', 'parent_post_rel_link', 10, 0 ); // remove parent post link
				remove_action( 'wp_head', 'adjacent_posts_rel_link', 10, 0 ); // remove the next and previous post links
				remove_action( 'wp_head', 'adjacent_posts_rel_link_wp_head', 10, 0 );

				remove_action( 'wp_head', 'print_emoji_detection_script', 7 );
				remove_action( 'wp_print_styles', 'print_emoji_styles' );

				remove_action( 'wp_head', 'wp_shortlink_wp_head', 10, 0 ); // Remove shortlink

				// Remove the REST API lines from the HTML Header
				remove_action( 'wp_head', 'rest_output_link_wp_head', 10 );
				remove_action( 'wp_head', 'wp_oembed_add_discovery_links', 10 );

				// Remove the REST API endpoint.
				remove_action( 'rest_api_init', 'wp_oembed_register_route' );

				// Turn off oEmbed auto discovery.
				add_filter( 'embed_oembed_discover', '__return_false' );

				// Don't filter oEmbed results.
				remove_filter( 'oembed_dataparse', 'wp_filter_oembed_result', 10 );

				// Remove oEmbed discovery links.
				remove_action( 'wp_head', 'wp_oembed_add_discovery_links' );

				// Remove oEmbed-specific JavaScript from the front-end and back-end.
				remove_action( 'wp_head', 'wp_oembed_add_host_js' );

				// Remove all embeds rewrite rules.
				add_action( 'rewrite_rules_array', array( $this, 'disable_embeds_rewrites' ), -1 );

				// Filters for WP-API version 1.x
				add_filter( 'json_enabled', '__return_false' );
				add_filter( 'json_jsonp_enabled', '__return_false' );

				// Filters for WP-API version 2.x
				add_filter( 'rest_enabled', '__return_false' );
				add_filter( 'rest_jsonp_enabled', '__return_false' );

				return GUTENTOR_PRO_PATH . 'templates/template-gutentor-popup.php';
			}

			return $template;
		}

		function disable_embeds_rewrites( $rules ) {
			foreach ( $rules as $rule => $rewrite ) {
				if ( false !== strpos( $rewrite, 'embed=true' ) ) {
					unset( $rules[ $rule ] );
				}
			}
			return $rules;
		}

		public function get_condition() {
			$condition                  = array();
			$condition['is_front_page'] = is_front_page();
			$condition['is_home']       = is_home();
			$condition['is_archive']    = is_archive();

			$condition['is_post_type_archive'] = is_post_type_archive();
			if ( is_post_type_archive() ) {
				$condition['post_type_archive'] = get_query_var( 'post_type' );
			}

			$condition['is_tax_archive'] = is_tax();
			if ( is_tax() ) {
				$tax                      = get_queried_object();
				$condition['tax_archive'] = $tax->term_id;
			}

			$condition['is_author_archive'] = is_author();
			if ( is_author() ) {
				$author                      = get_queried_object();
				$condition['author_archive'] = $author->ID;
			}

			$condition['is_singular'] = is_singular();
			if ( is_singular() ) {
				$condition['post_type'] = get_post_type();
				$condition['post_id']   = get_the_ID();
			}
			return $condition;
		}

		public function add_localize_data( $localize_data ) {
			$localize_data['is_admin']        = is_admin();
			$localize_data['is_on_adv_popup'] = gutentor_get_options( 'on-popup' );
			$localize_data['adv_popup_load']  = gutentor_get_options( 'popup-load' );

			if ( 'ajax' == $localize_data['adv_popup_load'] ) {
				$localize_data['condition'] = $this->get_condition();
			}

			return $localize_data;

		}

		private function is_true( $value ) {
			if ( $value == 'false' || $value == false ) {
				return false;
			}
			return true;
		}

		public function is_include_popup( $page_conditions, $conditions ) {
			$is_include = true;
			if ( is_array( $conditions ) && ! empty( $conditions ) ) {

				if ( count( $conditions ) === 1 ) {
					foreach ( $conditions as $condition ) {
						$include = isset( $condition['ie'] ) && ! empty( $condition['ie'] ) ? $condition['ie'] : 'include';
						$scope   = isset( $condition['scope'] ) && ! empty( $condition['scope'] ) ? $condition['scope'] : 'all';
						if ( $scope !== 'all' ) {
							if ( 'include' === $include ) {
								$add_condition = array(
									'ie'    => 'exclude',
									'scope' => 'all',
								);
							} else {
								$add_condition = array(
									'ie'    => 'include',
									'scope' => 'all',
								);
							}
							array_unshift( $conditions, $add_condition );
						}
					}
				}
				foreach ( $conditions as $condition ) {
					$include = isset( $condition['ie'] ) && ! empty( $condition['ie'] ) ? $condition['ie'] : 'include';
					$scope   = isset( $condition['scope'] ) && ! empty( $condition['scope'] ) ? $condition['scope'] : 'all';

					switch ( $scope ) {
						case 'all':
							if ( 'include' === $include ) {
								$is_include = true;
							} else {
								$is_include = false;
							}
							break;
						case 'front':
							if ( $this->is_true( $page_conditions['is_front_page'] ) ) {
								if ( 'include' === $include ) {
									$is_include = true;
								} else {
									$is_include = false;
								}
							}
							break;
						case 'home':
							if ( $this->is_true( $page_conditions['is_home'] ) ) {
								if ( 'include' === $include ) {
									$is_include = true;
								} else {
									$is_include = false;
								}
							}
							break;
						case 'archive':
							if ( $this->is_true( $page_conditions['is_archive'] ) ) {
								if ( 'include' === $include ) {
									$is_include = true;
								} else {
									$is_include = false;
								}
							}
							break;

						case 'post':
							if ( $this->is_true( $page_conditions['is_post_type_archive'] ) ) {
								$post_type = isset( $condition['post'] ) && ! empty( $condition['post'] ) ? $condition['post'] : array();
								if ( empty( $post_type ) ) {
									if ( 'include' === $include ) {
										$is_include = true;
									} else {
										$is_include = false;
									}
								} else {
									if ( is_array( $post_type ) && in_array( $page_conditions['post_type_archive'], $post_type ) ) {
										if ( 'include' === $include ) {
											$is_include = true;
										} else {
											$is_include = false;
										}
									}
								}
							}
							break;
						case 'taxonomy':
							if ( $this->is_true( $page_conditions['is_tax_archive'] ) ) {
								$tax = isset( $condition['tax'] ) && ! empty( $condition['tax'] ) ? $condition['tax'] : array();
								if ( empty( $tax ) ) {
									if ( 'include' === $include ) {
										$is_include = true;
									} else {
										$is_include = false;
									}
								} else {
									if ( is_array( $tax ) && in_array( $page_conditions['tax_archive'], $tax ) ) {
										if ( 'include' === $include ) {
											$is_include = true;
										} else {
											$is_include = false;
										}
									}
								}
							}
							break;

						case 'author':
							if ( $this->is_true( $page_conditions['is_author_archive'] ) ) {
								$author = isset( $condition['author'] ) && ! empty( $condition['author'] ) ? $condition['author'] : array();
								if ( empty( $tax ) ) {
									if ( 'include' === $include ) {
										$is_include = true;
									} else {
										$is_include = false;
									}
								} else {
									if ( is_array( $author ) && in_array( $page_conditions['author_archive'], $author ) ) {
										if ( 'include' === $include ) {
											$is_include = true;
										} else {
											$is_include = false;
										}
									}
								}
							}
							break;

						case 'singular':
							if ( $this->is_true( $page_conditions['is_singular'] ) ) {
								$post_type = isset( $condition['singular'] ) && ! empty( $condition['singular'] ) ? $condition['singular'] : array();
								$post_ids  = isset( $condition['id'] ) && ! empty( $condition['id'] ) ? $condition['id'] : '0';
								if ( empty( $post_type ) ) {
									if ( '0' == $post_ids ) {
										if ( 'include' === $include ) {
											$is_include = true;
										} else {
											$is_include = false;
										}
									} else {
										$post_ids = explode( ',', $post_ids );
										if ( is_array( $post_ids ) && in_array( $page_conditions['post_id'], $post_ids ) ) {
											if ( 'include' === $include ) {
												$is_include = true;
											} else {
												$is_include = false;
											}
										}
									}
								} else {
									if ( is_array( $post_type ) && in_array( $page_conditions['post_type'], $post_type ) ) {
										if ( '0' == $post_ids ) {
											if ( 'include' === $include ) {
												$is_include = true;
											} else {
												$is_include = false;
											}
										} else {
											$post_ids = explode( ',', $post_ids );
											if ( is_array( $post_ids ) && in_array( $page_conditions['post_id'], $post_ids ) ) {
												if ( 'include' === $include ) {
													$is_include = true;
												} else {
													$is_include = false;
												}
											}
										}
									}
								}
							}
							break;
					}

					if ( ! $is_include && 'all' != $scope ) {
						break;
					}
				}
			}

			return $is_include;
		}

		public function get_popup_data( $page_conditions ) {

			$args = array(
				'post_type'      => 'gutentor-template',
				'posts_per_page' => 100,
				'post_status'    => 'publish',

			);
			$filter_data = array();

			$the_query = new WP_Query( $args );
			if ( $the_query->have_posts() ) :
				while ( $the_query->have_posts() ) :
					$the_query->the_post();
					global $post;
					if ( ! empty( $post->post_content ) ) {
						/*First check conditions to include popup or not*/
						$conditions = get_post_meta( $post->ID, 'gutentor_p_condition', true );
						if ( ! gutentor_template()->is_include_popup( $page_conditions, $conditions ) ) {
							continue;
						}

						$type    = get_post_meta( $post->ID, 'gutentor_p_type', true );
						$trigger = get_post_meta( $post->ID, 'gutentor_p_trigger', true );

						$devices = get_post_meta( $post->ID, 'gutentor_p_show_device', true );
						$options = get_post_meta( $post->ID, 'gutentor_p_options', true );

						$data_attr = array();
						if ( isset( $type['type'] ) ) {
							$data_attr['type'] = $type['type'];
							if ( 'side' == $type['type'] ) {
								$data_attr['side']     = isset( $type['side'] ) ? $type['side'] : 'bottom-right';
								$data_attr['sideunit'] = isset( $type['sideUnit'] ) ? $type['sideUnit'] : 'px';
								$data_attr['x']        = isset( $type['x'] ) ? $type['x'] : '0';
								$data_attr['y']        = isset( $type['y'] ) ? $type['y'] : '0';
							}
							if ( 'side' == $type['type'] || 'modal' == $type['type'] ) {
								$data_attr['wunit'] = isset( $type['widthUnit'] ) ? $type['widthUnit'] : '%';
								$data_attr['wval']  = isset( $type['widthVal'] ) ? $type['widthVal'] : '70';

							}
						}

						if ( isset( $trigger['trigger'] ) ) {
							$data_attr['trigger'] = $trigger['trigger'];
							if ( ( 'load' == $trigger['trigger'] || 'idle' == $trigger['trigger'] ) && isset( $trigger['timing'] ) ) {
								$data_attr['timing'] = $trigger['timing'];
							}
							if ( 'scroll' == $trigger['trigger'] ) {
								$data_attr['direction'] = isset( $trigger['direction'] ) ? $trigger['direction'] : 'top';
								$data_attr['sunit']     = isset( $trigger['scrollUnit'] ) ? $trigger['scrollUnit'] : 'px';
								$data_attr['sval']      = isset( $trigger['scrollVal'] ) ? $trigger['scrollVal'] : '200';
							}
							if ( 'click' == $trigger['trigger'] ) {
								$data_attr['clickclass'] = $trigger['class'];
							}
						}
						if ( is_array( $devices ) && ! empty( $devices ) ) {
							$dval = array();
							foreach ( $devices as $device ) {
								$dval [] = $device['value'];
							}
							$data_attr['devices'] = json_encode( $dval );
						}
						$data_attr['animation']       = isset( $options['animation'] ) ? $options['animation'] : '';
						$data_attr['closeOnBgClick']  = isset( $options['closeOnBgClick'] ) && $options['closeOnBgClick'];
						$data_attr['closeBtnInside']  = ( isset( $options['closeBtnInside'] ) && $options['closeBtnInside'] );
						$data_attr['showCloseBtn']    = isset( $options['showCloseBtn'] ) && $options['showCloseBtn'];
						$data_attr['enableEscapeKey'] = isset( $options['enableEscapeKey'] ) && $options['enableEscapeKey'];
						$data_attr['fixedContentPos'] = isset( $options['fixedContentPos'] ) && $options['fixedContentPos'];
						$data_attr['fixedBgPos']      = isset( $options['fixedBgPos'] ) && $options['fixedBgPos'];
						$data_attr['closeBtnCol']     = isset( $options['closeBtnCol'] ) ? $options['closeBtnCol'] : '';
						$data_attr['bgType']          = isset( $options['bgType'] ) ? $options['bgType'] : '';
						$data_attr['bgCol']           = isset( $options['bgCol'] ) ? $options['bgCol'] : '';
						$data_attr['bgGradient']      = isset( $options['bgGradient'] ) ? $options['bgGradient'] : '';

						/*Popup Attributes*/
						$html_data = '';
						if ( ! empty( $data_attr ) ) {
							foreach ( $data_attr as $key => $val ) {
								$html_data .= $key . "='" . $val . "' ";
							}
						}

						/*Generate Popup HTML*/
						$popup_html  = "<div id='g-adv-popup-" . $post->ID . "' class='g-adv-popup mfp-hide' " . $html_data . '>';
						$popup_html .= "<div class='g-adv-popup-content'>";
						/*Get HTML from URL*/
						$url       = get_the_permalink();
						$body_args = array(
							/*API version*/
							'api_version' => wp_get_theme()['Version'],
							/*lang*/
							'site_lang'   => get_bloginfo( 'language' ),
						);
						$raw_json  = wp_safe_remote_get(
							$url,
							array(
								'timeout' => 100,
								'body'    => $body_args,
							)
						);

						if ( ! is_wp_error( $raw_json ) ) {
							$popup_html .= wp_remote_retrieve_body( $raw_json );
						}
						/*Get HTML from URL End*/
						$popup_html   .= '</div>';
						$filter_data[] = $popup_html;
					}

				endwhile;
				wp_reset_postdata();
			endif;
			return $filter_data;
		}

		public function add_footer_popup() {

			global $post;
			if ( $this->post_type === $post->post_type ) {
				return '';
			}
			if ( gutentor_get_options( 'on-popup' ) && 'ajax' != gutentor_get_options( 'popup-load' ) ) {
				$popup_data = $this->get_popup_data( $this->get_condition() );
				echo implode( '', $popup_data );/*ignore phpcs*/
			}
		}

	}
}

/**
 * Begins execution of the hooks.
 *
 * @since    1.0.0
 */
function gutentor_template() {
	return Gutentor_Template::instance();
}
gutentor_template()->run();