<?php
/**
 * The Gutentor theme hooks callback functionality of the plugin.
 *
 * @link       https://www.gutentor.com/
 * @since      1.0.0
 *
 * @package    Gutentor
 */

/**
 * The Gutentor theme hooks callback functionality of the plugin.
 *
 * Since Gutentor theme is hooks base theme, this file is main callback to add/remove/edit the functionality of the Gutentor Plugin
 *
 * @package    Gutentor
 * @author     Gutentor <info@gutentor.com>
 */
class Gutentor_Pro_Hooks {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {}

	/**
	 * Main Gutentor_Pro_Hooks Instance
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @return object $instance Gutentor_Pro_Hooks Instance
	 */
	public static function instance() {

		// Store the instance locally to avoid private static replication
		static $instance = null;

		// Only run these methods if they haven't been ran previously
		if ( null === $instance ) {
			$instance              = new Gutentor_Pro_Hooks();
			$instance->plugin_name = GUTENTOR_PRO_PLUGIN_NAME;
			$instance->version     = GUTENTOR_PRO_VERSION;
		}

		// Always return the instance
		return $instance;
	}

	/**
	 * Callback functions for admin_menu,
	 * Admin submenu page for Menu
	 *
	 * @since    1.0.0
	 * @access   public
	 *
	 * @param null
	 * @return void
	 */
	public function add_pro_menu() { // phpcs:ignore

		/*add_submenu_page(
			'gutentor',
			esc_html__( 'Gutentor Pro', 'gutentor' ),
			esc_html__( 'Pro', 'gutentor' ),
			'manage_options',
			'gutentor-pro',
			array( $this, 'pro_screen' )
		);*/
	}

	/**
	 * Display callback for the submenu page.
	 */
	function pro_screen() {
		?>
		<div class="wrap">
			<h1><?php esc_html_e( 'Gutentor Pro', 'gutentor' ); ?></h1>
			<p><?php esc_html_e( 'Thank you for installing Gutentor Pro', 'gutentor' ); ?></p>
		</div>
		<?php
	}

	/**
	 * Get Thumbnail all sizes.
	 *
	 * @since 2.0.0
	 */
	public static function get_thumbnail_all_sizes() {

		$sizes       = get_intermediate_image_sizes();
		$image_sizes = array();

		$image_sizes[] = array(
			'value' => 'full',
			'label' => esc_html__( 'Full', 'gutentor' ),
		);

		foreach ( $sizes as $size ) {
			if ( in_array( $size, array( 'thumbnail', 'medium', 'medium_large', 'large' ), true ) ) {
				$image_sizes[] = array(
					'value' => $size,
					'label' => ucwords( trim( str_replace( array( '-', '_' ), array( ' ', ' ' ), $size ) ) ),
				);
			}
		}
		return $image_sizes;
	}

	/**
	 * Get Thumbnail all sizes.
	 *
	 * @since 2.0.0
	 */
	public static function get_updated_thumbnail_all_sizes() {

		$sizes       = get_intermediate_image_sizes();
		$image_sizes = array();

		$image_sizes[] = array(
			'value' => 'full',
			'label' => esc_html__( 'Full', 'gutentor' ),
		);

		foreach ( $sizes as $size ) {
			$image_sizes[] = array(
				'value' => $size,
				'label' => ucwords( trim( str_replace( array( '-', '_' ), array( ' ', ' ' ), $size ) ) ),
			);
		}
		return $image_sizes;
	}

	/**
	 * Callback functions for enqueue_block_editor_assets,
	 * Enqueue Gutenberg block assets for backend only.
	 *
	 * @since    1.0.0
	 * @access   public
	 *
	 * @param null
	 * @return void
	 */
	public function block_editor_assets() { // phpcs:ignore

		// Scripts.
		wp_enqueue_script(
			'gutentor-pro-js', // Handle.
			plugins_url( '/build/blocks.js', dirname( __FILE__ ) ), // Block.build.js: We register the block here. Built with Webpack.
			array( 'lodash', 'wp-api', 'wp-i18n', 'wp-blocks', 'wp-components', 'wp-compose', 'wp-data', 'wp-editor', 'wp-edit-post', 'wp-element', 'wp-keycodes', 'wp-plugins', 'wp-rich-text', 'wp-viewport', 'gutentor' ), // Dependencies, defined above.
			'1.0.0', // Version: File modification time.
			true // Enqueue the script in the footer.
		);

		wp_localize_script(
			'gutentor-pro-js',
			'gutentorpro',
			array(
				'dirUrl'     => GUTENTOR_PRO_URL,
				'nonce'      => wp_create_nonce( 'gutentorNonce' ),
				'popoverSvg' => GUTENTOR_PRO_URL . 'assets/img/block-icons/popover-icon.svg',
				'wrapperSvg' => GUTENTOR_PRO_URL . 'assets/img/block-icons/wrapper-icon.svg',
				'setting'    => array(
					'videoInPt' => gutentor_setting_videos_in_pt(),
					'popup'     => gutentor_setting_enable_popup(),
				),
			)
		);
	}

	/**
	 * Callback functions for admin_enqueue_scripts,
	 * Enqueue Gutenberg block assets for backend editor.
	 *
	 * @since    1.0.0
	 * @access   public
	 *
	 * @param null
	 * @return void
	 */
    public function admin_resources() { // phpcs:ignore

		if ( gutentor_is_edit_page() ) {
		    /*For video upload*/
			wp_enqueue_script(
				'gutentor-admin', // Handle.
				GUTENTOR_URL . 'assets/js/admin-script' . GUTENTOR_SCRIPT_PREFIX . '.js', // Block.build.js: We register the block here. Built with Webpack.
				array( 'wp-color-picker' ), // Dependencies, defined above.
				GUTENTOR_VERSION, // Version: File modification time.
				true // Enqueue the script in the footer.
			);
		}
		$screen              = get_current_screen();
		$admin_scripts_bases = array( 'toplevel_page_gutentor', 'gutentor_page_gutentor-blocks', 'gutentor_page_gutentor-settings', 'term', 'edit-tags' );
		$admin_scripts_bases = apply_filters( 'gutentor_admin_scripts_current_screen', $admin_scripts_bases );
		if ( ! ( isset( $screen->base ) && in_array( $screen->base, $admin_scripts_bases ) ) ) {
			return;
		}
		// Scripts.
		wp_enqueue_script(
			'gutentor-pro-admin-build', // Handle.
			plugins_url( '/build/admin.js', dirname( __FILE__ ) ), // Block.build.js: We register the block here. Built with Webpack.
			array( 'gutentor-admin-build' ), // Dependencies, defined above.
			'1.0.0', // Version: File modification time.
			true // Enqueue the script in the footer.
		);
	}

	/**
	 * Callback functions for gutentor_is_load_resource,
	 * Condition to load or not.
	 *
	 * @since    1.0.0
	 * @access   public
	 *
	 * @param string load
	 * @param string resource
	 *
	 * @return void
	 */
	public function add_load_condition( $load, $resource ) {

		switch ( $resource ) {
			case 'magnificpopup':
				if ( gutentor_has_block( 'gutentor/p1' ) ) {
					$load = true;
				}
				break;
		}
		return $load;
	}

	/**
	 * Coming Soon (Maintenance) request
	 *
	 * @since    1.0.0
	 * @access   public
	 *
	 * @param array $query
	 * @return array
	 */

	public function redirect_to_maintenance() {

		if ( 'maintenance' === gutentor_get_options( 'maintenance-mode' ) ) {

			$access = gutentor_get_options( 'maintenance-access' );

			$template = gutentor_get_options( 'maintenance-template' );

			$redirect = true;
			if ( -1 != $template && get_the_ID() != $template ) {
				if ( 'none' === $access ) {
					$redirect = true;
				} elseif ( is_user_logged_in() ) {
					if ( 'logged-in' === $access ) {
						$redirect = false;
					}
					if ( 'admin' === $access && current_user_can( 'administrator' ) ) {
						$redirect = false;
					}
				}

				if ( $redirect ) {
					wp_redirect( get_the_permalink( $template ) );
					die;
				}
			}
		}
	}
}

/**
 * Begins execution of the hooks.
 *
 * @since    1.0.0
 */
function gutentor_pro_hooks() {
	return Gutentor_Pro_Hooks::instance();
}
