<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used to
 * add/remove/edit the functionality of the Gutentor Plugin
 *
 * @link       https://www.gutentor.com/
 * @since      1.0.0
 *
 * @package    Gutentor
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * functionality of the plugin
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Gutentor
 * @author     Gutentor <info@gutentor.com>
 */
class Gutentor_Pro {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Gutentor_Pro_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * Full Name of plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_full_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_full_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Main Instance
	 *
	 * Insures that only one instance of Gutentor_Pro exists in memory at any one
	 * time. Also prevents needing to define globals all over the place.
	 *
	 * @since    1.0.0
	 * @access   public
	 *
	 * @return object
	 */
	public static function instance() {

		// Store the instance locally to avoid private static replication
		static $instance = null;

		// Only run these methods if they haven't been ran previously
		if ( null === $instance ) {
			$instance = new Gutentor_Pro();
		}

		// Always return the instance
		return $instance;
	}
	/**
	 * Class constructor.
	 */
	public function __construct() {
		add_action( 'admin_notices', array( $this, 'gutentor_missing_notice' ) );
	}

	/**
	 * Since the plugin is created specially for Gutentor
	 * Check if Gutentor is active
	 *
	 * @since    1.0.0
	 */
	public function is_gutentor_active() {

		if ( function_exists( 'run_gutentor' ) ) {
			return true;
		}

		return false;
	}

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		if ( defined( 'GUTENTOR_PRO_VERSION' ) ) {
			$this->version = GUTENTOR_PRO_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = GUTENTOR_PRO_PLUGIN_NAME;

		$this->load_dependencies();
		$this->set_locale();

		$this->define_hooks();
		$this->load_hooks();

	}


	/**
	 * Since the plugin is created specially for Gutentor
	 * Show notice if Gutentor plugin is not installed/activated
	 *
	 * @since    1.0.0
	 */
	public function gutentor_missing_notice() {
		if ( $this->is_gutentor_active() ) {
			return;
		}

		/*
		TODO
		$url    = wp_nonce_url(
			add_query_arg(
				array(
					'action' => 'activate',
					'plugin' => WP_PLUGIN_DIR . '/gutentor/gutentor.php',
				),
				admin_url( 'plugins.php' )
			),
			'activate-plugin_' . WP_PLUGIN_DIR . '/gutentor/gutentor.php'
		);
		$action = sprintf(
			'<a href="%1$s" aria-label="%2$s">%2$s</a>',
			esc_url( $url ),
			__( 'Activate Gutentor' )
		);
		*/
		$search_url = in_array( 'gutentor/gutentor.php', array_keys( get_plugins() ), true ) ? admin_url( 'plugins.php?s=gutentor' ) : admin_url( 'plugin-install.php?s=gutentor' );

		echo '<div class="error notice is-dismissible"><p><strong>' . esc_html__( 'Gutentor Pro', 'gutentor-pro' ) . '</strong> &#8211; ' . sprintf( esc_html__( 'This plugin requires %s Plugin to be activated to work.', 'gutentor' ), '<a href="' . esc_url( $search_url ) . '">' . esc_html__( 'Gutentor', 'gutentor' ) . '</a>' ) . '</p></div>';
	}


	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Gutentor_Pro_Loader. Orchestrates the hooks of the plugin.
	 * - Gutentor_Pro_i18n. Defines internationalization functionality.
	 * - Gutentor_Pro_Admin. Defines all hooks for the admin area.
	 * - Gutentor_Pro_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once GUTENTOR_PRO_PATH . 'includes/loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once GUTENTOR_PRO_PATH . 'includes/i18n.php';

		/*library*/
		require_once GUTENTOR_PRO_PATH . 'includes/lib/lib.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		/*meta box*/
		require_once GUTENTOR_PRO_PATH . 'includes/hooks.php';
		require_once GUTENTOR_PRO_PATH . 'includes/functions.php';

		require_once GUTENTOR_PRO_PATH . 'includes/gutentor-template/gutentor-template.php';

		require_once GUTENTOR_PRO_PATH . 'includes/metabox/meta-box.php';

		require_once GUTENTOR_PRO_PATH . 'includes/tools/class-gutentor-extend-api.php';

		require_once GUTENTOR_PRO_PATH . 'includes/block-base/class-gutentor-extend-setting.php';

		require_once GUTENTOR_PRO_PATH . 'includes/block-base/class-gutentor-featured-img-video-hooks.php';
		require_once GUTENTOR_PRO_PATH . 'includes/block-base/class-gutentor-featured-img-popup-hooks.php';

		require_once GUTENTOR_PRO_PATH . 'includes/block-base/class-p1-hooks.php';
		require_once GUTENTOR_PRO_PATH . 'includes/block-base/class-p1-timeline.php';
		/*blog template*/
		require_once GUTENTOR_PRO_PATH . 'includes/block-template/blog/class-blog-query-elements.php';
		require_once GUTENTOR_PRO_PATH . 'includes/block-template/blog/class-blog-templates.php';

		/*Dynamic Content*/
		require_once GUTENTOR_PRO_PATH . 'includes/dynamic-content.php';

		/*Custom Fonts*/
		require_once GUTENTOR_PRO_PATH . 'includes/custom-fonts.php';

		/*Custom Icons*/
		require_once GUTENTOR_PRO_PATH . 'includes/custom-icons.php';

		$this->loader = new Gutentor_Pro_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Gutentor_Pro_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Gutentor_Pro_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_hooks() {

		$plugin_hooks = gutentor_pro_hooks();

		/*Hook: Editor assets.*/
		$this->loader->add_action( 'enqueue_block_editor_assets', $plugin_hooks, 'block_editor_assets', 1999 );

		/*Admin Assets*/
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_hooks, 'admin_resources', 1999 );

		/*Add Menu*/
		$this->loader->add_action( 'admin_menu', $plugin_hooks, 'add_pro_menu' );

		/*Add Gutentor Load Condition*/
		$this->loader->add_filter( 'gutentor_is_load_resource', $plugin_hooks, 'add_load_condition', 100, 2 );

		/*Coming Soon (Maintenance) request */
		$this->loader->add_filter( 'template_redirect', $plugin_hooks, 'redirect_to_maintenance', 99999, 1 );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function load_hooks() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Gutentor_Pro_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
