<?php
/**
 * Custom Meta
 *
 * @package gutentor
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Gutentor_Pro_Custom_Meta_Box' ) ) :

	class Gutentor_Pro_Custom_Meta_Box {

		/**
		 * Main Instance
		 *
		 * Insures that only one instance of Gutentor_Pro_Custom_Meta_Box exists in memory at any one
		 * time. Also prevents needing to define globals all over the place.
		 *
		 * @since    1.0.0
		 * @access   public
		 *
		 * @return object
		 */
		public static function instance() {

			// Store the instance locally to avoid private static replication
			static $instance = null;

			// Only run these methods if they haven't been ran previously
			if ( null === $instance ) {
				$instance = new Gutentor_Pro_Custom_Meta_Box();
			}

			// Always return the instance
			return $instance;
		}

		/**
		 *  Run functionality with hooks
		 *
		 * @since    1.0.0
		 * @access   public
		 *
		 * @return void
		 */
		public function run() {

			if ( is_admin() ) {
				add_action( 'load-post.php', array( $this, 'init_metabox' ) );
				add_action( 'load-post-new.php', array( $this, 'init_metabox' ) );
			}

		}
		/**
		 * Meta box initialization.
		 */
		public function init_metabox() {
			add_action( 'add_meta_boxes', array( $this, 'add_metabox' ) );
			add_action( 'save_post', array( $this, 'save_metabox' ), 10, 2 );
		}

		/**
		 * Adds the meta box.
		 */
		public function add_metabox() {
			$all_post_type   = array( 'post', 'page' );
			$get_postType    = gutentor_admin_get_post_types();
			$access_postType = gutentor_setting_videos_in_pt();

			if ( $get_postType ) {
				foreach ( $get_postType as $type ) {
					array_push( $all_post_type, $type['value'] );
				}
			}
			if ( is_array( $access_postType ) ) {
				foreach ( $all_post_type as $type ) {
					if ( in_array( $type, $access_postType ) ) {
						add_meta_box(
							'gutentor_meta_video_src_option',
							__( 'Gutentor Settings', 'gutentor' ),
							array( $this, 'render_metabox' ),
							$type,
							'side',
							'low'
						);
					}
				}
			}

		}

		/**
		 * Renders the meta box.
		 */
		public function render_metabox( $post ) {

			$g_video_src_options      = array(
				'external-url' => esc_html__( 'External Url', 'gutentor' ),
				'self-hosted'  => esc_html__( 'Self Hosted (Local)', 'gutentor' ),
			);
			$g_video_src_selected_val = get_post_meta( $post->ID, 'gutentor_meta_video_src_option', true );
			$g_video_url              = get_post_meta( $post->ID, 'gutentor_meta_video_url', true );
			$g_video_id               = get_post_meta( $post->ID, 'gutentor_meta_video_id', true );
			// true ensures you get just one value instead of an array
			wp_nonce_field( basename( __FILE__ ), 'gutentor_pro_meta_nonce' );

			?>
				<style>
					.gutentor-video-preview-wrap {
						display: block;
						margin-bottom: 5px;
					}
				</style>
			<div class="gutentor-video-custom-meta components-base-control">
				<div class="components-base-control__field">
					<label class="components-base-control__label"><?php echo esc_html__( 'Video Source', 'gutentor' ); ?></label>
					<select name="gutentor_meta_video_src_option" id="gutentor_meta_video_src_option"
							class="components-select-control__input" style="width:100%;">
                        <!--inline style width 100% is added in components-select-control__input because default is not loaded in download page -->
						<?php foreach ( $g_video_src_options as $key => $value ) { ?>
							<option value="<?php echo esc_attr( $key ); ?>" <?php selected( $key, $g_video_src_selected_val ); ?>><?php echo $value; ?></option>
						<?php } // end foreach ?>
					</select>
				</div>
				<div class="gutentor-meta-video-wrap">
					<div class="form-table page-meta-box">
						<div class="header_video">
							<label class="description" for="gutentor_meta_video_url">
								<?php esc_html_e( 'Enter Video URL', 'gutentor' ); ?>
							</label>
							<input id="gutentor_meta_video_url" type="text" class="widefat"
								   name="gutentor_meta_video_url"
								   value="<?php echo esc_url( $g_video_url ); ?>"/>
							<span class="gutentor-video-support-notice">
								<?php esc_html_e( 'Note: Youtube video url only supported.', 'gutentor' ); ?>
							</span>
						</div>
						<div class="gutentor-meta-video-upload-wrap">
							<!--video -->
							<p>
								<label for="gutentor-media-video-upload">
									<?php esc_html_e( 'Upload Video', 'gutentor' ); ?>
								</label>

								<span class="gutentor-video-preview-wrap">
							 <?php
								if ( ! empty( $g_video_id ) ) {
									$g_upload_video_url = wp_get_attachment_url( $g_video_id );
									if ( $g_upload_video_url ) {
										?>
									 <video class="gutentor-video-preview" src="<?php echo esc_url( $g_upload_video_url ); ?>" type='video/mp4' controls="true"></video>
										<?php
									}
								}
								?>
							</span><!-- .gutentor-video-preview-wrap -->
								<input type="hidden" class="widefat" name="gutentor_meta_video_id"
									   id="gutentor_meta_video_id"
									   value="<?php echo absint( $g_video_id ); ?>"/>
								<input type="button" value="<?php esc_attr_e( 'Upload Video', 'gutentor' ); ?>"
									   class="button gutentor-media-video-upload"
									   data-title="<?php esc_attr_e( 'Select Video', 'gutentor' ); ?>"
									   data-button="<?php esc_attr_e( 'Select Video', 'gutentor' ); ?>"/>
								<input type="button" value="<?php esc_attr_e( 'Remove Video', 'gutentor' ); ?>"
									   class="button gutentor-media-video-remove"/>
							</p>
							<span class="gutentor-video-support-notice">
								<?php esc_html_e( 'Upload your video in format', 'gutentor' ); ?>
								<code><?php esc_html_e( '.mp4', 'gutentor' ); ?></code>
								<?php esc_html_e( 'and minimize its file size for best results.', 'gutentor' ); ?>
							</span>
							<!--video -->
						</div>
					</div>
				</div>
				<?php do_action( 'gutentor_meta_fields', $post ); ?>
			</div>
			<?php
		}


		/**
		 * Handles saving the meta box.
		 *
		 * @param int     $post_id Post ID.
		 * @param WP_Post $post    Post object.
		 * @return null
		 */
		public function save_metabox( $post_id, $post ) {

			/*
			  * A Guide to Writing Secure Themes – Part 4: Securing Post Meta
			  *https://make.wordpress.org/themes/2015/06/09/a-guide-to-writing-secure-themes-part-4-securing-post-meta/
			  * */
			if (
				! isset( $_POST['gutentor_pro_meta_nonce'] ) ||
				! wp_verify_nonce( $_POST['gutentor_pro_meta_nonce'], basename( __FILE__ ) ) || /*Protecting against unwanted requests*/
				( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) || /*Dealing with autosaves*/
				! current_user_can( 'edit_post', $post_id )/*Verifying access rights*/
			) {
				return;
			}

			if ( 'page' == $_POST['post_type'] ) {
				if ( ! current_user_can( 'edit_page', $post_id ) ) {
					return $post_id;
				}
			} elseif ( ! current_user_can( 'edit_post', $post_id ) ) {
				return $post_id;
			}

			// Execute this saving function
			// Video layout
			if ( isset( $_POST['gutentor_meta_video_src_option'] ) ) {
				$old = get_post_meta( $post_id, 'gutentor_meta_video_src_option', true );
				$new = esc_attr( $_POST['gutentor_meta_video_src_option'] );
				if ( $new && $new != $old ) {
					update_post_meta( $post_id, 'gutentor_meta_video_src_option', $new );
				} elseif ( '' == $new && $old ) {
					delete_post_meta( $post_id, 'gutentor_meta_video_src_option', $old );
				}
			}

			if ( isset( $_POST['gutentor_meta_video_url'] ) ) {
				$old = get_post_meta( $post_id, 'gutentor_meta_video_url', true );
				$new = esc_url_raw( $_POST['gutentor_meta_video_url'] );
				if ( $new && $new != $old ) {
					update_post_meta( $post_id, 'gutentor_meta_video_url', $new );
				} elseif ( '' == $new && $old ) {
					delete_post_meta( $post_id, 'gutentor_meta_video_url', $old );
				}
			}

			if ( isset( $_POST['gutentor_meta_video_id'] ) ) {
				$old = get_post_meta( $post_id, 'gutentor_meta_video_id', true );
				$new = absint( $_POST['gutentor_meta_video_id'] );
				if ( $new && $new != $old ) {
					update_post_meta( $post_id, 'gutentor_meta_video_id', $new );
				} elseif ( '' == $new && $old ) {
					delete_post_meta( $post_id, 'gutentor_meta_video_id', $old );
				}
			}

			do_action( 'gutentor_save_meta_fields', $post_id, $post, $_POST );
		}

	}

endif;

/**
 * Create Instance for Gutentor_Pro_Custom_Meta_Box
 *
 * @since    1.0.0
 * @access   public
 *
 * @param
 * @return object
 */
if ( ! function_exists( 'gutentor_pro_custom_meta_box' ) ) {

	function gutentor_pro_custom_meta_box() {

		return Gutentor_Pro_Custom_Meta_Box::instance();
	}

	gutentor_pro_custom_meta_box()->run();
}
